(* ===== SCRIPT PROPERTIES ===== *)
property thePassword : ""
property theLength : 8 --Set the length of the password here
property containsLetters : true
property containsNumbers : true
property bulet : ""
property isBullets : ""
property randomCase : false


(* ===== GLOBAL SCRIPT VARIABLES ===== *)
global GenerateButton, numberBox, letterBox, bulletBox, caseBox


(* ===== EVENT HANDLERS ===== *)
--Launch event handler
on launched theObject
	scriptInitialize()
end launched
--Click event handler
on clicked theObject
	--Number checkbox event handler
	if theObject is numberBox then
		numberRoutine()
	end if
	--Letter checkbox event handler
	if theObject is letterBox then
		letterRoutine()
	end if
	--Random case checkbox event handler
	if theObject is caseBox then
		caseCheck()
	end if
	--Bullet checkbox event handler
	if theObject is bulletBox then
		bulletCheck()
	end if
	--Password result pane button event handlers
	if title of theObject is "OK" then
		closeWindow(window of theObject)
	end if
	if title of theObject is "Copy" then
		clipboardCopy(theObject)
	end if
	if title of theObject is "Clear Clipboard" then
		clearClipboard()
	end if
	if title of theObject is "Another" then --User wants another password generated
		anotherRoutine(theObject)
	end if
	if theObject is GenerateButton then
		initializeGenerator() --Checks for errors
		bulletCheck() --Checks if bullet box is checked
		generatePassword() --Generate the password
		displayResults() --Display the results
	end if
end clicked
on should quit after last window closed theObject
	quit
end should quit after last window closed


(* ===== SUBROUTINES ===== *)
(* ===== Initialization Routines ===== *)
on scriptInitialize() --Script initialization
	--Setup UI element names
	set GenerateButton to button "Generate" of window "Main"
	set numberBox to button "Numbers" of window "Main"
	set letterBox to button "Letters" of window "Main"
	set bulletBox to button "Bullet" of window "Main"
	set caseBox to button "Randomcase" of window "Main"
end scriptInitialize
on initializeGenerator()
	try
		set theLength to contents of text field "Length" of window "Main" as number
		set theLength to theLength as integer
		(* ===== ERROR ROUTINES ===== *)
	on error number the_number
		if the_number is -1700 then
			display alert "The password length must be a positive integer between 1 and 24" attached to window "Main" as critical
			return
		end if
	end try
	if theLength is greater than 24 then
		display alert "The password length cannot exceed twenty-four characters." attached to window "Main" as critical
		return
	end if
	if theLength  0 then
		display alert "Please enter a valid password length." attached to window "Main" as critical
		return
	end if
end initializeGenerator

(* ===== Checking Routines ===== *)
on checkBoxes() --Another error checking routine
	if state of numberBox is 0 then --Numbers box isn't checked
		if state of letterBox is 0 then --Letters checkbox isn't checked as well
			enableButton(false, GenerateButton) --Disable the Generate button
		else if state of letterBox is 1 then
			enableButton(true, GenerateButton)
		end if
	else if state of letterBox is 0 then --Letters checkbox isn't checked
		if state of numberBox is 0 then --Numbers box isn't checked
			enableButton(false, GenerateButton) --Disable the Generate button
		else if state of numberBox is 1 then
			enableButton(true, GenerateButton)
		end if
	end if
end checkBoxes
on bulletCheck()
	if state of button "Bullet" of window "main" is 1 then
		--Button is on
		set bulet to ""
		repeat theLength times
			set bulet to bulet & ""
		end repeat
		set isBullets to 1
		return bulet
	else if state of button "Bullet" of window "Main" is 0 then
		set isBullets to 0
	end if
end bulletCheck
on numberRoutine()
	if state of numberBox is equal to 0 then --Box is not checked
		checkBoxes() --Call box checking routine
		set containsNumbers to false
	else
		set containsNumbers to true
		checkBoxes() --Call box checking routine
	end if
end numberRoutine
on letterRoutine()
	if state of letterBox is equal to 0 then --Box is not checked
		enableButton(false, caseBox) --Disable the case button by calling the enableButton routine
		set containsLetters to false
		checkBoxes() --Call box checking routine
	else if state of letterBox is equal to 1 then --Box is checked, enable random case check box
		set containsLetters to true
		enableButton(true, caseBox) --Enables the random case button
		checkBoxes() --Call box checking routine
	end if
end letterRoutine
on caseCheck()
	if state of caseBox is 1 then --Box is checked
		set randomCase to true
	else if state of caseBox is 0 then --Box is unchecked
		set randomCase to false
	end if
end caseCheck

(* ===== Password generating routine ===== *)
on generatePassword()
	set thePassword to "" --Clear password variable
	(* Password character generation routine *)
	repeat theLength times
		--Let's check what the user wants
		if (containsNumbers is true) and (containsLetters is false) then --User wants to have numbers in the password
			set characterChoice to 1 --1 is the number character choice
		else if (containsNumbers is false) and (containsLetters is true) then --User does not want numbers in the password, but wants letters.
			set characterChoice to 0 --0 is the letter character choice
		else if (containsNumbers is true) and (containsLetters is true) then --User wants both numbers and letters in the password
			set characterChoice to random number from 0 to 1 --To randomize whether a number or a letter is chosen to add to the password this time through the loop, use a random number from 0 to 1, basically giving us a random boolean.
		end if
		(* ===== Letter generating routine ===== *)
		if characterChoice is 0 then --This time a random letter will be generated
			set realLetter to generateLetter(randomCase)
			set thePassword to thePassword & realLetter
		end if
		(* ===== Number generating routine ===== *)
		if characterChoice is 1 then --This time a random number will be generated
			set theNum to generateNumber()
			set thePassword to thePassword & theNum
		end if
	end repeat
end generatePassword
on displayResults()
	set thePanel to load panel "PasswordPane" from nib "PasswordPane"
	if isBullets is 1 then --User wants bullets
		tell thePanel
			set contents of text field "Password" to bulet
		end tell
	else if isBullets is 0 then
		tell thePanel
			set contents of text field "Password" to thePassword
		end tell
	end if
	display panel thePanel attached to window "Main"
end displayResults

(* ===== Character generation subroutine ===== *)
on generateNumber() --Number generator
	set theNum to random number from 0 to 9
	return theNum
end generateNumber
on generateLetter(value) --Letter generator
	if value is true then --User wants randomCase
		set randomTemp to random number from 0 to 1 --Use randomTemp to determine character case this time through the loop.
		if randomTemp is 0 then --Lower case letter will be generated
			set letterList to {"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"}
		else if randomTemp is 1 then --Uppercase letter will be generated
			set letterList to {"A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"}
		end if
		set randomTemp to random number from 1 to 26 --26 letters in the alphabet right?
		set realLetter to item randomTemp of letterList
		return realLetter --Finished with random case generating routine, return to parent
	else if randomCase is false then --User does not want random case, so generated lower case
		set letterList to {"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"}
		set randomTemp to random number from 1 to 26 --26 letters in the alphabet right?
		set realLetter to item randomTemp of letterList
		return realLetter --Finished with lower case generating routine, return to parent
	end if
end generateLetter
on generateSymbol()
end generateSymbol




(* ===== Password result pane subroutines ===== *)
on clearClipboard()
	set the clipboard to "" as string
end clearClipboard

on clipboardCopy(theObject)
	set passw to thePassword as string
	set the clipboard to passw
	closeWindow(window of theObject)
	set thePanel to load panel "Clipboard" from nib "Clipboard"
	display panel thePanel attached to window "Main"
end clipboardCopy

on anotherRoutine(theObject)
	set theLength to contents of text field "Length" of window "Main" as number
	generatePassword() --Call password generating script
	if isBullets is 0 then --No bullets
		tell window of theObject
			set contents of text field "Password" to thePassword
		end tell
	end if
end anotherRoutine

(* ===== General purpose subroutines ===== *)
on enableButton(value, theButton)
	set enabled of theButton to value
end enableButton
on closeWindow(theWindow)
	close panel (theWindow)
end closeWindow